@@ -43,7 +43,7 @@ class Fiber():
 
     def xyFocalPlaneSAB(self, pixperrow):
         """
-        :return east and up location in units of mm for Sciene IFU or background
+        :return east and up location in units of um for Sciene IFU or background
         """
         idx = int(self.name[3:])
         # Note that there is no need to check the upper index
@@ -146,6 +146,7 @@ class Fiber():
         # location of x=east, y=up of P2-1 or P1-1
         x = self.SQRT3_2 * row
         y = -0.5* row + idx
+        r = math.hypot(x,y)
         # angle relative to 0=up, east=90
         baseAng = math.atan2(x,y)
         # for odd 0-based indices use 60 deg = pi/3 rad complement
@@ -157,7 +158,7 @@ class Fiber():
         # the rotation angle increases by 60 deg clockwise.
         rot60 = fidx // 2
         baseAng += self.PI_3*rot60
-        return self.pitch*math.sin(baseAng), self.pitch*math.cos(baseAng)
+        return self.pitch*r*math.sin(baseAng), self.pitch*r*math.cos(baseAng)
 
     def xyFocalPlane(self):
         """
@@ -185,3 +186,59 @@ class Fiber():
         e, up = self.xyFocalPlane()
         # return (math.pi/2.0 -math.atan2(up,e)) # worse: might be outside +-180deg
         return math.atan2(e,up)
+
+    def dump_gnuplot(self,idx) :
+        x, y = self.xyFocalPlane()
+        print(x, " ", y, " ", self.pitch*0.2)
+        # print(x, " ", y)
+        print("# set label \"", self.name[1:] , "\" at ", x, ",",y)
+        # print("# set label \"", idx , "\" at ", x, ",",y)
+
+
+    @staticmethod
+    def dump_gnuplot_frame() :
+        # add a reference frame 2200x3200 of a FLIR color 4.5 um camera
+        # that's 9900 x 14400 um.
+        print("# set arrow 1 from -4950,-7200 to 4950,-7200")
+        print("# set arrow 2 from 4950,-7200 to 4950,7200")
+        print("# set arrow 3 from 4950,7200 to -4950,7200")
+        print("# set arrow 4 from -4950,7200 to -4950,-7200")
+        print("# set arrow 1 nohead lw 4")
+        print("# set arrow 2 nohead lw 4")
+        print("# set arrow 3 nohead lw 4")
+        print("# set arrow 4 nohead lw 4")
+        print("\n\n")
+
+    @staticmethod
+    def dump_gnuplot_bund(bundles):
+        """
+        :param bundles: a list of fiber bundle names (each name 3 letters ending -)
+        :type bundles: a list of str
+        """
+        for bundle in bundles :
+            for idx in range(602):
+                name = bundle+str(idx)
+                try:
+                    fib = Fiber(name)
+                    fib.dump_gnuplot(idx)
+                except Exception as e:
+                    pass  # index out of range
+                    # print(idx,str(e))
+        Fiber.dump_gnuplot_frame()
+
+#     @staticmethod
+#     def dump_gnuplot_all():
+#         """ Dump the circle locations (in microns) to stdout
+#         suitable for a gnuplot plotting with "set style data circle"
+#         The result may be run thru gnuplot, then convert of ImageMagick
+#         to generate FITS files of the the fiber bundles in the focal plane.
+# 
+#         from lvmtipo import Fiber
+#         fiber.dump_gnuplot() > fib.dat
+#         fiber.dump_gnuplot() | fgrep "#" | tr "#" " " > fib.gp
+#         convert fib.png fib.fits
+#         """
+#         Fiber.dump_gnuplot( [ 'S1-','S2-','S3-'] )
+#         Fiber.dump_gnuplot( [ 'A1-','A2-','A3-'] )
+#         Fiber.dump_gnuplot( [ 'B1-','B2-','B3-'] )
+#         Fiber.dump_gnuplot( [ 'P1-','P2-'] )
